<?php

/** Logger class
 *
 * @author Claire Figueras <dev@maarch.org>
 **/
class Logger
{
	/** 
	 * Current handlers (file, console, etc...) 
     *
     * @protected
     **/
	protected $handlers = array();
	
	
	/** 
	 * Array of errors levels
     *
     * @protected
     **/
	protected $error_levels = array('DEBUG' => 0, 'INFO' => 2, 'WARNING' => 2, 'ERROR' => 3);
	
	/** 
	 * Minimum error level
     *
     * @protected
     **/
	protected $threshold_level;
	
	/** Class constructor
     *
     * Inits the threshold level
     * 
	 * @param $threshold_level (string) Threshold level (set to 'INFO' by default)
     **/
	function __construct($threshold_level = 'ERROR')
	{
		$this->threshold_level = $threshold_level;
	}
	
	/** Writes error message in current handlers
	 * 
	 * writes only if the error level is greater or equal the threshold level
     * 
	 * @param $msg (string) Error message
	 * @param $error_level (string) Error level (set to 'INFO' by default)
	 * @param $error_code (integer) Error code (set to 0 by default)
     **/
	public function write($msg, $error_level = 'ERROR', $error_code = 0, $other_params = array())
	{
		if(!array_key_exists($error_level, $this->error_levels))
		{
			$error_level = 'ERROR';
		}	
		if($this->error_levels[$error_level] >= $this->error_levels[$this->threshold_level])
		{
			for($i=0; $i<count($this->handlers);$i++)
			{
				$this->handlers[$i]->write($msg, $error_level, $error_code, $other_params);
			}
		}
	}
	
	/** Adds a new handler in the current handlers array
	 * 
	 * @param $handler (object) Handler object
     **/
	public function add_handler($handler)
	{
		if(!isset($handler))
			return false;
		
		array_push($this->handlers, $handler);
		return true;
	}
	
	/** Sets treshold level
	 * 
	 * @param $treshold (string) treshold level
     **/
	public function set_threshold_level($treshold)
	{
		if(isset($treshold) && array_key_exists($treshold, $this->error_levels))
		{
			$this->threshold_level = $treshold;
			return true;
		}
		$this->threshold_level = 'INFO';
		return false;
	}
	
	/** Removes a given handler of the current handlers array and destructs it 
	 * 
	 * @param $handler (object) Handler object
     **/
	public function remove_handler($handler)
	{
		if(!isset($handler))
			return false;
		
		for($i=0; $i<count($this->handlers);$i++)
		{
			if($this->handlers[$i] === $handler_id)
			{
				$this->handlers[$i]->__destruct();
				unset($this->handlers[$i]);
				break;
			}
		}
		$this->handlers = array_values($this->handlers);
		return true;
	}
	
	/** Class destructor 
	 * 
	 * Calls handlers destructors
     **/
	function __destruct()
	{
		for($i=0; $i<count($this->handlers);$i++)
		{
			$this->handlers[$i]->__destruct();
		}
	}
}
?>
