<?php
require_once 'PHPUnit/Framework.php';
require_once 'Maarch_CLITools/ArgsParser.php';

class ArgsParserForTest extends ArgsParser {
    function &__get($key) {
        if (property_exists($this, $key)) {
            return $this->$key;
        }
        return null;
    }
    function __set($key, $value) {
        $this->$key = $value;
    }
    
    function __call($name, $arguments) {
        return call_user_func_array(Array($this, $name), $arguments);
    }
}


class ArgsParserTest extends PHPUnit_Framework_TestCase{
    public function setUp(){
        $this->parser = new ArgsParserForTest();
        $this->assertTrue($this->parser instanceof ArgsParser);
    }
    public function tearDown(){
        unset($this->parser);
        $this->assertNull($this->parser);
    }


    public function testInitialisation(){
        $this->assertObjectHasAttribute("_options", $this->parser);
        $this->assertAttributeSame(Array("positional"=>Array()), "_options", $this->parser);

        $this->assertObjectHasAttribute("_mapping", $this->parser);
        $this->assertAttributeSame(Array(), "_mapping", $this->parser);

        $this->assertObjectHasAttribute("_args", $this->parser);
        $this->assertAttributeSame(Array(), "_args", $this->parser);

        $this->assertObjectHasAttribute("_positionals", $this->parser);
        $this->assertAttributeSame(Array(), "_positionals", $this->parser);

        $this->assertObjectHasAttribute("_name", $this->parser);
        $this->assertAttributeSame("__main__", "_name", $this->parser);

    }

    function testAddSwitchNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddSwitchOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", "q");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
    }

    function testAddSwitchOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddSwitchShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "switch", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddSwitchOnlyCatalog(){
        $this->parser->add_arg("quiet", "switch", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddSwitchAll(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("switch", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->_mapping));
    }

    function testAddArgumentNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddArgumentOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", "q");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
    }

    function testAddArgumentOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddArgumentShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "argument", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertArrayHasKey('q', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["q"]);
        $this->assertArrayHasKey('quiet', $this->parser->_mapping);
        $this->assertSame("quiet", $this->parser->_mapping["quiet"]);
    }

    function testAddArgumentOnlyCatalog(){
        $this->parser->add_arg("quiet", "argument", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
    }

    function testAddArgumentAll(){
        $this->parser->add_arg("quiet", "argument", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame("q", $this->parser->_args["quiet"]["short"]);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["long"]);
        $this->assertSame("argument", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(2, count($this->parser->_mapping));
    }

    function testAddPositionalNoInfo(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddPositionalOnlyShort(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", "q");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddPositionalOnlyLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", null, "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddPositionalShortLong(){
        #add_arg($name, $type, $short=null, $long=null, $default=null, $mandatory=false, $help="")
        $this->parser->add_arg("quiet", "positional", "q", "quiet");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(false, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddPositionalOnlyCatalog(){
        $this->parser->add_arg("quiet", "positional", null, null, true, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddPositionalAll(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", 42, true, "my help");
        $this->assertArrayHasKey('quiet', $this->parser->_args);
        $this->assertSame("quiet", $this->parser->_args["quiet"]["name"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["short"]);
        $this->assertSame(null, $this->parser->_args["quiet"]["long"]);
        $this->assertSame("positional", $this->parser->_args["quiet"]["type"]);
        $this->assertSame(42, $this->parser->_args["quiet"]["default"]);
        $this->assertSame(true, $this->parser->_args["quiet"]["mandatory"]);
        $this->assertSame("my help", $this->parser->_args["quiet"]["help"]);
        $this->assertEquals(0, count($this->parser->_mapping));
        $this->assertSame(Array("quiet"), $this->parser->_positionals);
    }

    function testAddCommandMinimal(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array(), $subparser);
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->_args);
        $this->assertSame('sub', $this->parser->_args['sub']['name']);
        $this->assertSame('command', $this->parser->_args['sub']['type']);
        $this->assertSame('', $this->parser->_args['sub']['help']);
        $this->assertSame(Array(), $this->parser->_args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->_args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub"]);
    }

    function testAddCommandWithAliases(){
        #add_command($name, $aliases, &$parser, $help="")
        $subparser = new ArgsParserForTest();
        $this->parser->add_command("sub", Array("sub2", "sub3"), $subparser, "my help");
        $this->assertSame("sub", $subparser->name);
        $this->assertArrayHasKey('sub', $this->parser->_args);
        $this->assertSame('sub', $this->parser->_args['sub']['name']);
        $this->assertSame('command', $this->parser->_args['sub']['type']);
        $this->assertSame("my help", $this->parser->_args['sub']['help']);
        $this->assertSame(Array("sub2", "sub3"), $this->parser->_args['sub']['aliases']);
        $this->assertSame($subparser, $this->parser->_args['sub']['parser']);
        $this->assertArrayHasKey('sub', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub"]);
        $this->assertArrayHasKey('sub2', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub2"]);
        $this->assertArrayHasKey('sub3', $this->parser->_mapping);
        $this->assertSame("sub", $this->parser->_mapping["sub3"]);
    }
    
    function testCheckMandatoryArgDefaultNotGiven(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", 42, true, "my help");
        $this->assertTrue($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatoryArgDefaultGiven(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", 42, true, "my help");
        $this->parser->_options['quiet'] = '27';
        $this->assertTrue($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatoryArgNoDefaultNotGiven(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", null, true, "my help");
        $this->assertFalse($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatoryArgNoDefaultGiven(){
        $this->parser->add_arg("quiet", "positional", "q", "quiet", null, true, "my help");
        $this->parser->_options['quiet'] = '27';
        $this->assertTrue($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatorySwitchDefaultNotGiven(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", true, true, "my help");
        $this->assertTrue($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatorySwitchDefaultGiven(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", true, true, "my help");
        $this->parser->_options['quiet'] = false;
        $this->assertTrue($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatorySwitchNoDefaultNotGiven(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", null, true, "my help");
        $this->assertFalse($this->parser->_check_mandatory_args());
    }
    
    function testCheckMandatorySwitchNoDefaultGiven(){
        $this->parser->add_arg("quiet", "switch", "q", "quiet", null, true, "my help");
        $this->parser->_options['quiet'] = true;
        $this->assertTrue($this->parser->_check_mandatory_args());
    }

}